{$I-,S-}
program DumpHasp;
uses misc,strings,dos;
CONST
   ver = '1.21';
(*
 * A list of the HASP services
 *)
   IS_HASP                      =  1;
   GET_HASP_CODE                =  2;
   READ_MEMO                    =  3;
   WRITE_MEMO                   =  4;
   GET_HASP_STATUS              =  5;
   GET_ID_NUM                   =  6;
   READ_MEMO_BLOCK              = 50;
   WRITE_MEMO_BLOCK             = 51;

(*
 * A list of the TimeHASP services
 *)
   TIMEHASP_SET_TIME            = 70;
   TIMEHASP_GET_TIME            = 71;
   TIMEHASP_SET_DATE            = 72;
   TIMEHASP_GET_DATE            = 73;
   TIMEHASP_WRITE_MEMORY        = 74;
   TIMEHASP_READ_MEMORY         = 75;
   TIMEHASP_WRITE_MEMORY_BLOCK  = 76;
   TIMEHASP_READ_MEMORY_BLOCK   = 77;
   TIMEHASP_GET_ID_NUM          = 78;

(*
 * A list of NetHASP services.
 *)
   NET_LAST_STATUS              = 40;
   NET_GET_HASP_CODE            = 41;
   NET_LOGIN                    = 42;
   NET_LOGOUT                   = 43;
   NET_READ_WORD                = 44;
   NET_WRITE_WORD               = 45;
   NET_GET_ID_NUMBER            = 46;
   NET_READ_MEMO_BLOCK          = 52;
   NET_WRITE_MEMO_BLOCK         = 53;

   OK                           = 0;
   NET_READ_ERROR               = 131;
   NET_WRITE_ERROR              = 132;


(*
 * A list of LptNum codes for different key types
 *)
   LPT_IBM_ALL_HASP25  = 0;
   LPT_IBM_ALL_HASP36  = 50;
   LPT_NEC_ALL_HASP36  = 60;

   MEMO_BUFFER_SIZE  = 56;
   MAX_COUNT = $FFFF;
{$L hasptp.obj}
{$F+}
procedure  hasp (Service, SeedCode, LptNum, Pass1, Pass2 : integer;
                 var p1,p2,p3,p4 : integer);external;
{$F-}

(*
 * Global variables.
 *)
Type
_HASPDumpHdr = record
   _hdr :string[8];     	{'HASPDump'}
   _date:string[16];    	{date}

   _type:word;			{hasp type}
   _misc:longint;
  _pwdHi:word;
  _pwdLo:word;
   _sn  :longint;		{serial nu}
   _misc1:string[5];
   _bf  :array [0..MEMO_BUFFER_SIZE] of word;
end;


var
   LptNum : integer;
   Pass1            : integer;
   Pass2            : integer;
   p1,p2,p3,p4      : integer;
   Service          : integer;
   ID               : longint;
   i                : integer;
   DmpHdr	    : _HASPDumpHdr;
   k,m: integer;
   SeedCode:word;
   HaspMemoBuff     : array[1..MEMO_BUFFER_SIZE] of word;
   ProductType      : integer;
   f:text;
   f1:file;

function AsciiDot(tt:byte):char;
begin
if ((tt>$1F) and (tt<$B0)) or ((tt>$DF) and (tt<$F0)) then AsciiDot:=chr(tt)
else AsciiDot:='.';
end;

(*
 ***************************************
 *
 *  DumpHASPMemory;
 *
 *  Description: Dumps HASP-3 Memory
 *
 ***************************************
 *)
procedure WriteHASPBlk;
begin
    p1 := 0;                     {memory start address}
    p2 := MEMO_BUFFER_SIZE;      {block length}
    p3 := Seg(HaspMemoBuff);
    p4 := Ofs(HaspMemoBuff);
    Service := WRITE_MEMO_BLOCK;
    hasp ( Service, SeedCode, LptNum, Pass1, Pass2, p1, p2, p3, p4 );
    if p3 <> 0 then
        writeln('Write Block operation failed. Error number: ',p3)
    else
        writeln('Write Block operation succeeded.');
end;


procedure DumpHASPMemory;
begin
(*
 * Read Memory Block
 *)
    for i := 1 to MEMO_BUFFER_SIZE do
        HaspMemoBuff[i] := 0;

    writeln('Reading a block from HASP memory...');

    p1 := 0;                     {memory start address}
    p2 := MEMO_BUFFER_SIZE;      {block length}
    p3 := Seg(HaspMemoBuff);
    p4 := Ofs(HaspMemoBuff);
    Service := READ_MEMO_BLOCK;
    hasp ( Service, SeedCode, LptNum, Pass1, Pass2, p1, p2, p3, p4 );
    if p3 <> 0 then begin
       writeln('Read Block operation failed. Error number: ',p3);
       halt ($ff);
       end
    else

      begin
        writeln('Read Block operation succeeded.');
        writeln('Block values are : ');
        k:=0;
        i:=1;
        while i< MEMO_BUFFER_SIZE+1 do
           begin
            write(' ', hexb(Lo(HaspMemoBuff[i]))+' '+hexb(Hi(HaspMemoBuff[i])));
            inc(i);
            inc(k);
             if (k div 8) =1 then
               begin
                 write ('  ');
                 for m:=i-k to i-1 do begin
                    write (AsciiDot(Lo(HaspMemoBuff[m]))+AsciiDot(Hi(HaspMemoBuff[m])));
                 end;
                k:=0;
                writeln;
               end;
           end;
      end;
end;

(*
 ***************************************
 *
 *  DumpHASPSeedCodes;
 *
 *  Description: Dumps HASP-3 Seed codes
 *
 ***************************************
 *)
   procedure DumpHASPSeedCodes;
var
buf1 : array [1..4] of word;
begin

(*
 * Get HASP code
 *)
    writeln('Getting HASP code...');
    Service := GET_HASP_CODE;
 For SeedCode:=0 to MAX_COUNT do begin
    hasp ( Service, SeedCode, LptNum, Pass1, Pass2, p1, p2, p3, p4 );
    writex (#13+'Retrieved: '+hexw(SeedCode)+#0);
{    writeln('Seed Code is: ',hexw(SeedCode),'  Return: ',
              hexw(p1),' ',hexw(p2),' ',hexw(p3),' ',hexw(p4));
}
    buf1[1]:=p1;
    buf1[2]:=p2;
    buf1[3]:=p3;
    buf1[4]:=p4;
    blockwrite(f1, buf1[1], sizeof(buf1));
    end;
end;

function GetHASPID:longint;
begin

(*
 * Read the ID number.
 * The ID number is a 32 bit integer which is calculated as follows:
 * ID = p1 + 65536 * p2
 *)
    writeln('Getting the ID number of the MemoHASP key...');
    p1 := 0;
    p2 := 0;
    Service := GET_ID_NUM;
    hasp ( Service, SeedCode, LptNum, Pass1, Pass2, p1, p2, p3, p4 );
    if p3 <> 0 then begin
        writeln('Get ID Number operation failed. Error number: ',p3);
        halt($ff);
        end
    else
      begin
        ID := word(p2);
        ID := ID shl 16;
        ID := ID + word(p1);
        GetHASPID:=ID;
      end;

end;

BEGIN     { main program body }

if paramcount>0 then begin
   assign(output, paramstr(1));
   rewrite(output);
 end;

   writeln;
   writeln( ' HASP memory dumper v',ver,', (x)1997 by MeteO');


(*
 * Get the passwords.
 *)

    writeX( 'Enter the first HASP password  : '+#0 );
    readln( Pass1 );
    writeX( 'Enter the second HASP password : '+#0 );
    readln( Pass2 );


 writeln;
{
 Pass1:=$3c39;
 pass2:=$25a0;

 Pass1:=$1eb3;
 pass2:=$7a75;

 Pass1:=$2f6b;
 pass2:=$233b;

 Pass1:=$576d;
 pass2:=$2eab;
}
(*
 * Determine the key type.
 *)
    LptNum := LPT_IBM_ALL_HASP25;
(*
 * Check if any HASP is connected
 *)
    writeln('Checking if any HASP is connected...');
    Service := IS_HASP;
    hasp ( Service, SeedCode, LptNum, Pass1, Pass2, p1, p2, p3, p4 );
    if p1 = 0 then
    begin
      writeln('No HASP key was found.');
      exit;
    end
    else
      writeln('A HASP key was found.');

   writeln('The HASP ID Number is: ', hexd(GetHASPId));

 with DmpHdr do
   begin
      _hdr := 'HASP'+' '+ver;
      _date:= AsciiTime;
      _sn  := GetHASPId;
     _pwdHi:= Pass1;
     _pwdLo:= Pass2;
   end;

   DumpHASPMemory;
   for i:=1 to MEMO_BUFFER_SIZE do DmpHdr._bf[i]:=HaspMemoBuff[i];
   assign (f1, hexw(Pass1)+hexw(Pass2)+'.dmp');
   rewrite(f1,1);
   blockwrite(f1,DmpHdr,sizeof(DmpHdr));

  DumpHASPSeedCodes;

END.

